@@ -154,7 +154,7 @@ read_all_makefiles (makefiles)
 	while (d->next != tail)
 	  d = d->next;
 
-	/* Use the storage read_filename allocates.  */
+	/* Use the storage read_makefile allocates.  */
 	free (*makefiles);
 	*makefiles = dep_name (d);
 	++num_makefiles;
@@ -623,6 +623,9 @@ read_makefile (filename, type)
 	  if (two_colon)
 	    p2++;
 
+	  /* We have some targets, so don't ignore the following commands.  */
+	  no_targets = 0;
+
 	  /* Is this a static pattern rule: `target: %targ: %dep; ...'?  */
 	  p = index (p2, ':');
 	  while (p != 0 && p[-1] == '\\')
